###############################################################
#
#   SuPy - Menu utilities
#
###############################################################

debug = False

#
#   Bit values returned by menu item validation proc
#
DISABLE = 1
CHECK = 8

menus = {}                  #  name --> top level Menu
context_menu_handlers = {}  #  name --> handler

keep('menus', 'context_menu_handlers')


class Menu(object):
	#  name      string
	#  rb_menu   UI.Menu
	#  items     {label : MenuItem}
	
	"""
	Representation of a Sketchup menu.
	
	Attributes:
	  name      Name of the menu, prefixed with parent menu name if a submenu
	  rb_menu   Corresponding UI::Menu instance
	  items     Dict mapping item names to MenuItems or other Menus
	"""

	def __init__(self, name, rb_menu):
		self.name = name
		self.rb_menu = rb_menu
		self.items = {}
		self.need_separator = False
	
	def __str__(self):
		return "<Sketchup Menu %r>" % self.name
	
	def add_item(self, label, action = None, validator = None, body = None):
		"""
		If the menu does not already have an item with the given
		label, adds a new item and sets its action and validation
		procedures; otherwise, replaces the procedures of the existing
		item. Returns the new or existing MenuItem.
		"""
		item = self.items.get(label)
		if not item:
			self._new_item()
			if debug:
				print "Menu: Materialising item", label
			item = MenuItem(self.rb_menu, label)
			self.items[label] = item
		if not isinstance(item, MenuItem):
			raise ValueError("Menu item %r was previously a %s" % 
				(label, item.__class__.__name__))
		item.action = action or body
		item.validator = validator
		return item
	
	def add_submenu(self, label):
		item = self.items.get(label)
		if not item:
			self._new_item()
			if debug:
				print "Menu: Materialising submenu", label
			rb_submenu = self.rb_menu.add_submenu(label)
			item = Menu(label, rb_submenu)
			self.items[label] = item
		#print "menus.Menu.add_submenu: doing isinstance(%r, %r)" % (item, Menu) ###
		if not isinstance(item, Menu):
			raise ValueError("Menu item %r was previously a %s" % 
				(label, item.__class__.__name__))
		return item
	
	def add_separator(self):
		self.need_separator = True
	
	def _new_item(self):
		if self.need_separator:
			if debug:
				print "Menu: Materialising separator"
			self.rb_menu.add_separator()
			self.need_separator = False


class MenuItem(object):
	#  item_id     int
	#  action      func()
	#  validator   func() -> boolean

	"""
	Representation of a Sketchup menu item.
	
	Attributes:
	  item_id     The Sketchup menu item id
	  action      Action function: func()
	  validator   Validator function: func() -> boolean
	"""
	
	action = None
	validator = None
	
	def __str__(self):
		return "<Sketchup MenuItem %s>" % self.item_id
	
	def __init__(self, rb_menu, label):
		self.item_id = rb_menu.add_item(label, body = self.call_action)
		val = self.call_validator
		rb_menu.set_validation_proc(self.item_id, body = val)
	
	def call_action(self):
		proc = self.action
		if proc:
			proc()
	
	def call_validator(self):
		proc = self.validator
		if proc:
			ret = proc()
			return ret
		else:
			return 0


def get_menu(name):
	"""
	Return the Menu instance for the specified top-level menu.
	"""
	menu = menus.get(name)
	if not menu:
		rb_menu = UI.menu(name)
		menu = Menu(name, rb_menu)
		menus[name] = menu
	return menu

def add_context_menu_handler(handler, name = None):
	"""
	Installs the given context menu handler under the given name. If a handler
	has already been installed under the same name, replaces the previous one.
	
	If no name is specified, the handler should be a function, and the name
	defaults to <modname>.<funcname>, where <modname> is the fully-qualified
	name of the module where the function is defined, and <funcname> is the
	name of the function.
	
	The handler is passed a Menu instance representing the contextual menu
	to be updated.
	"""
	if name:
		key = name
	else:
		key = _find_handler_key(handler)
		if not key:
			raise ValueError("Unable to find module and name of %r and no handler name given" % handler)
	if key not in context_menu_handlers:
		def f(menu):
			h = context_menu_handlers.get(key)
			if h:
				h(Menu("", menu))
		UI.add_context_menu_handler(body = f)
	context_menu_handlers[key] = handler

def _find_handler_key(function):
	g = getattr(function, 'func_globals', None)
	if not g:
		return None
	m = g.get("__name__")
	if not m:
		return None
	n = getattr(function, 'func_name')
	if not n:
		return None
	return "%s.%s" % (m, n)
